/***************************************************************************
                          block.cpp  -  description
                             -------------------
    begin                : Fri Feb 11 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivan_m_r@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "block.h"

#include "loop.h"
#include "input.h"

CBlock::CBlock()
{
	NameA = "";
}
CBlock::~CBlock()
{
}
/**  */
void CBlock::Arrange(void* ParameterL = NULL)
{
	if (IllegalCIdentifier(Name()))
		AddError(500);
	else
	{
		SetIdentifier("block_" + Container()->Identifier() + "_" + Name());
	}
}
/**  */
bool CBlock::SetName(const string& NameL)
{
	NameA = NameL;

	if (IllegalCIdentifier(NameL))
	{
		AddError(500);
		return false;
	}	
	else
		return true;
}
/**  */
const string& CBlock::Name() const
{
	return NameA;
}
/**  */
bool CBlock::DescendantOf(const string& ClassNameL) const
{
	if (ClassNameL == "CBlock")
		return true;
	else
		return CCodeGenerator::DescendantOf(ClassNameL);
}
/**  */
void CBlock::GenerateCall(ostream& StreamL, void* ExtraParameterL = NULL) const
{
	StreamL << "      " << Identifier() << "();" << endl;
}
/**  */
bool CBlock::CandidateForOrdenation(const CCodeGeneratorPointers& OrderedComponents) const
{
	CCodeGeneratorPointers::ConstIterator ComponentPIL;
	bool InputsInitializedOrFromOrderedL;
	CInput* InputVariablePL;

	if (!OrderedComponents.Contains(this))
	{
		ComponentPIL = Components().Begin();

		InputsInitializedOrFromOrderedL = true;
		while ((ComponentPIL != Components().End()) && InputsInitializedOrFromOrderedL)
		{
			if ((*ComponentPIL)->DescendantOf("CInput"))
			{
				InputVariablePL = (CInput*)(*ComponentPIL);

				InputsInitializedOrFromOrderedL = InputVariablePL->Initialized() ||
					InputVariablePL->ConnectedToOrderedBlock(OrderedComponents);
			}
			ComponentPIL++;
		}
		return InputsInitializedOrFromOrderedL;
	}
	else
		return false;
}
/**  */
const CLoop* CBlock::Container() const
{
	return (CLoop*)CCodeGenerator::Container();
}
/**  */
void CBlock::ComponentsDependingArrange(void* ExtraParameterL = NULL)
{
	if (Container()->RepeatedValue(this, "name", Name()))
		AddError(501);
}
/**  */
bool CBlock::RepeatedValue(const CCodeGenerator* CallerPL,
	const string& ValueNameL, const string& ValueL) const
{
	if ((CallerPL != this) && (ValueNameL == "name") && (ValueL == Name()))
		return true;
	else
		return Components().RepeatedValue(CallerPL, ValueNameL, ValueL);
}
/**  */
bool CBlock::AcceptsContainer(const CCodeGenerator* pContainer)
{
	if (pContainer->DescendantOf("CLoop"))
	{
		SetContainer(pContainer);
		return true;
	}
	else
	{
		AddError(101);
		return false;
	}
}
