/***************************************************************************
                          codegenerator.cpp  -  description
                             -------------------
    begin                : Fri May 12 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivan_m_r@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "codegenerator.h"

#include <strstream>

CCodeGenerator::CCodeGenerator()
{
	m_Components.Clear();
	m_FixedComponents.Clear();
	m_Errors.Clear();
	ContainerPA = NULL;
	IdentifierA = "";
}
CCodeGenerator::~CCodeGenerator()
{
}
/**  */
void CCodeGenerator::SetIdentifier(const string& IdentifierL)
{
	IdentifierA = IdentifierL;
}
/**  */
const string& CCodeGenerator::Identifier() const
{
	return IdentifierA;
}
/**  */
void CCodeGenerator::AllErrorsReport(CErrors& ErrorsReport) const
{
	Components().AllErrorsReport(ErrorsReport);
	FixedComponents().AllErrorsReport(ErrorsReport);
	ErrorsReport.Append(Errors());
}
/**  */
void CCodeGenerator::Arrange(void* ExtraParameterL = NULL)
{
	if (IllegalCIdentifier(Identifier()))
		AddError(100);
}
/**  */
CCodeGeneratorPointers& CCodeGenerator::Components()
{
	return m_Components;
}
/**  */
const CCodeGeneratorPointers& CCodeGenerator::Components() const
{
	return m_Components;
}
/**  */
bool CCodeGenerator::AddComponent(CCodeGenerator* pComponent)
{
	strstream Stream;

	if (pComponent->AcceptsContainer(this))
	{
		Components().PushBack(pComponent);
		return true;
	}
	else
	{
		Errors().Append(pComponent->Errors());
		AddError(102);
		return false;
	}
}
/**  */
void CCodeGenerator::SetContainer(const CCodeGenerator* ContainerL)
{
	ContainerPA = ContainerL;
}
/**  */
const CCodeGenerator* CCodeGenerator::Container() const
{
	return ContainerPA;
}
/**  */
void CCodeGenerator::DeleteObjects()
{
	Components().DeleteObjects();
	FixedComponents().DeleteObjects();
}
/**  */
bool CCodeGenerator::DescendantOf(const string& ClassNameL) const
{
	if (ClassNameL == "CCodeGenerator")
		return true;
	else
		return false;
}
/**  */
void CCodeGenerator::GenerateCode(CStringsSet& StringsSetL,
	const CCodeGenerator* CallerPL, const string& PartL, void* ExtraParameterL = NULL) const
{
}
/**  */
void CCodeGenerator::ComponentsDependingArrange(void* ExtraParameterL = NULL)
{
}
/**  */
bool CCodeGenerator::IllegalCIdentifier(const string& IdentifierL) const
{
  string::const_iterator CharacterI;

  CharacterI = IdentifierL.begin();
  if (((*CharacterI >= 'a') && (*CharacterI <= 'z')) ||
  		((*CharacterI >= 'A') && (*CharacterI <= 'Z')) || (*CharacterI == '_'))
	  do
	    CharacterI++;
	  while ((CharacterI != IdentifierL.end()) &&
	  	(((*CharacterI >= 'a') && (*CharacterI <= 'z')) ||
	  	((*CharacterI >= 'A') && (*CharacterI <= 'Z')) ||
	  	((*CharacterI >= '0') && (*CharacterI <= '9')) || (*CharacterI == '_')));
	if (IdentifierL.empty() || (CharacterI != IdentifierL.end()))
		return true;
	else
		return false;
}
/**  */
CErrors& CCodeGenerator::Errors()
{
	return m_Errors;
}
/**  */
const CErrors& CCodeGenerator::Errors() const
{
	return m_Errors;
}
/**  */
void CCodeGenerator::AllClearErrors()
{
	Errors().Clear();
	Components().ClearAllErrors();
	FixedComponents().ClearAllErrors();
}
/**  */
bool CCodeGenerator::RepeatedValue(const CCodeGenerator* CallerPL, const string& ValueNameL,
	const string& ValueL) const
{
	return Components().RepeatedValue(CallerPL, ValueNameL, ValueL) ||
		FixedComponents().RepeatedValue(CallerPL, ValueNameL, ValueL);
}
/**  */
bool CCodeGenerator::ErrorsReported() const
{
	return !Errors().Empty();
}
/**  */
void CCodeGenerator::AllGenerateCode(ostream& StreamL,
	const CCodeGenerator* CallerPL,	const string& SectionL,
	void* ExtraParameterL = NULL) const
{
	ComponentsGenerateCode(StreamL, CallerPL, SectionL, ExtraParameterL);
	GenerateCode(StreamL, CallerPL, SectionL, ExtraParameterL);
}
/**  */
bool CCodeGenerator::AllArrange(void* ExtraParameterL = NULL)
{
	Arrange(ExtraParameterL);
	if (ErrorsReported())
		return false;
	else
		if (Components().Arrange(ExtraParameterL) &&
			FixedComponents().Arrange(ExtraParameterL))
		{
			ComponentsDependingArrange(ExtraParameterL);
			return !ErrorsReported();
	}
		else
			return false;
}
/**  */
void CCodeGenerator::AllGenerateCode(CStringsSet& StringsSetL,
	const CCodeGenerator* CallerPL, const string& SectionL,
	void* ExtraParameterL = NULL) const
{
	ComponentsGenerateCode(StringsSetL, CallerPL, SectionL, ExtraParameterL);
	GenerateCode(StringsSetL, CallerPL, SectionL, ExtraParameterL);
}
/**  */
void CCodeGenerator::ComponentsGenerateCode(ostream& StreamL,
	const string& SectionL, void* ExtraParameterL = NULL) const
{
	Components().GenerateCode(StreamL, this, SectionL, ExtraParameterL);
	FixedComponents().GenerateCode(StreamL, this, SectionL, ExtraParameterL);
}
/**  */
void CCodeGenerator::ComponentsGenerateCode(CStringsSet& StringsSetL,
	const string& SectionL, void* ExtraParameterL = NULL) const
{
	Components().GenerateCode(StringsSetL, this, SectionL, ExtraParameterL);
	FixedComponents().GenerateCode(StringsSetL, this, SectionL, ExtraParameterL);
}
/**  */
unsigned int CCodeGenerator::MaximumInComponents() const
{
	return 0;
}
/**  */
void CCodeGenerator::AddError(unsigned int uNumber, const CCodeGenerator* pLocation)
{
	CError Error;
	Error.SetLocation(pLocation);
	Error.SetNumber(uNumber);
	Errors().PushBack(Error);
}
/**  */
const CCodeGeneratorPointers& CCodeGenerator::FixedComponents() const
{
	return m_FixedComponents;
}
/**  */
CCodeGeneratorPointers& CCodeGenerator::FixedComponents()
{
	return m_FixedComponents;
}
/**  */
void CCodeGenerator::AddError(unsigned int uNumber)
{
	CError Error;
	Error.SetLocation(this);
	Error.SetNumber(uNumber);
	Errors().PushBack(Error);
}
/**  */
void CCodeGenerator::ComponentsGenerateCode(ostream& Stream,
	const CCodeGenerator* pCaller, const string& Section, void* pExtraParameter = NULL) const
{
	Components().GenerateCode(Stream, pCaller, Section, pExtraParameter);
	FixedComponents().GenerateCode(Stream, pCaller, Section, pExtraParameter);
}
/**  */
void CCodeGenerator::ComponentsGenerateCode(CStringsSet& StringsSet,
	const CCodeGenerator* pCaller, const string& Section, void* pExtraParameter = NULL) const
{
	Components().GenerateCode(StringsSet, pCaller, Section, pExtraParameter);
	FixedComponents().GenerateCode(StringsSet, pCaller, Section, pExtraParameter);
}
