/***************************************************************************
                          codegeneratorch  -  description
                             -------------------
    begin                : Fri May 12 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivan_m_r@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CODEGENERATOR_H
#define CODEGENERATOR_H

/**
  *@author Ivan Martinez
  */

#include <string>
#include <iostream>
#include <set>
#include "codegeneratorpointers.h"
#include "errors.h"

class CCodeGenerator {
public:
  /**  */
  typedef set<string,less<string> > CStringsSet;
  /**  */
	CCodeGenerator();
  /**  */
	virtual ~CCodeGenerator();
	/**  */
	virtual bool AcceptsContainer(const CCodeGenerator*) = 0;
  /**  */
	bool AddComponent(CCodeGenerator*);
  /**  */
  void AllErrorsReport(CErrors&) const;
  /**  */
  bool AllArrange(void* = NULL);
  /**  */
  virtual string Class() const = 0;
  /**  */
  void AllClearErrors();
  /**  */
  const CCodeGeneratorPointers& Components() const;
  /**  */
  void ComponentsGenerateCode(ostream&, const CCodeGenerator*, const string&,
		void* = NULL) const;
  /**  */
  void ComponentsGenerateCode(CStringsSet&, const CCodeGenerator*, const string&,
		void* = NULL) const;
  /**  */
  const CCodeGenerator* Container() const;
  /**  */
  void DeleteObjects();
  /**  */
  virtual bool DescendantOf(const string&) const;
	/**  */
  bool ErrorsReported() const;
  /**  */
  const CErrors& Errors() const;
  /**  */
  const CCodeGeneratorPointers& FixedComponents() const;
  /**  */
  void AllGenerateCode(ostream&, const CCodeGenerator*, const string&,
		void* = NULL) const;
  /**  */
  void AllGenerateCode(CStringsSet&, const CCodeGenerator*,
		const string&, void* = NULL) const;
  /**  */
  const string& Identifier() const;
  /**  */
  virtual unsigned int MaximumInComponents() const;
  /**  */
  virtual bool RepeatedValue(const CCodeGenerator*, const string&, const string&) const;
protected:
  /**  */
  void AddError(unsigned int);
  /**  */
  void AddError(unsigned int, const CCodeGenerator*);
  /**  */
  virtual void Arrange(void* = NULL);
  /**  */
  CCodeGeneratorPointers& Components();
  /**  */
  virtual void ComponentsDependingArrange(void* = NULL);
  /**  */
  void ComponentsGenerateCode(ostream&, const string&, void* = NULL) const;
  /**  */
  void ComponentsGenerateCode(CStringsSet&, const string&, void* = NULL) const;
  /**  */
  CErrors& Errors();
  /**  */
  CCodeGeneratorPointers& FixedComponents();
  /**  */
  virtual void GenerateCode(ostream&, const CCodeGenerator*, const string&,
		void* = NULL) const = 0;
  /**  */
  virtual void GenerateCode(CStringsSet&, const CCodeGenerator*, const string&,
		void* = NULL) const;
  /**  */
  bool IllegalCIdentifier(const string&) const;
  /**  */
  void SetContainer(const CCodeGenerator*);
  /**  */
  void SetIdentifier(const string&);
private: // Private attributes
  /**  */
  string IdentifierA;
  /**  */
  CCodeGeneratorPointers m_Components;
  /**  */
  CCodeGeneratorPointers m_FixedComponents;
  /**  */
  const CCodeGenerator* ContainerPA;
  /**  */
  CErrors m_Errors;
};

#endif
