/***************************************************************************
                          dynamicsystem.cpp  -  description
                             -------------------
    begin                : Thu Feb 10 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivan_m_r@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dynamicsystem.h"

#include "function.h"
#include <strstream>

CDynamicSystem::CDynamicSystem()
{
	m_llTime = 0;
}
CDynamicSystem::~CDynamicSystem()
{
}
/**  */
bool CDynamicSystem::SetTime(long long llTime)
{
	m_llTime = llTime;

	if (llTime > 0)
		return true;
	else
	{
		AddError(900);
		return false;
	}
}
/**  */
string CDynamicSystem::Class() const
{
	return "CDynamicSystem";
}
/**  */
bool CDynamicSystem::DescendantOf(const string& ClassNameL) const
{
	if (ClassNameL == "CDynamicSystem")
		return true;
	else
		return CRTModule::DescendantOf(ClassNameL);
}
/**  */
void CDynamicSystem::GenerateSource(ostream& StreamL, void* ExtraParameterL = NULL) const
{
	CCodeGenerator::CStringsSet StringsSetL;
	
	StringsSetL.insert("#include <rtl.h>");
	StringsSetL.insert("#include <rtl_time.h>");

	ComponentsGenerateCode(StringsSetL, "includes", ExtraParameterL);
	copy(StringsSetL.begin(), StringsSetL.end(), ostream_iterator<string>(StreamL, "\n"));

	ComponentsGenerateCode(StreamL, "types declaration", ExtraParameterL);

	StreamL << "hrtime_t start_time;" << endl;
	StreamL << "char run;" << endl;

	StringsSetL.clear();
	ComponentsGenerateCode(StringsSetL, "global variables declaration",
		ExtraParameterL);
	copy(StringsSetL.begin(), StringsSetL.end(), ostream_iterator<string>(StreamL, "\n"));

	ComponentsGenerateCode(StreamL, "global variables declaration",
		ExtraParameterL);
	ComponentsGenerateCode(StreamL, "threads declaration", ExtraParameterL);
	ComponentsGenerateCode(StreamL, "fifos declaration", ExtraParameterL);
	ComponentsGenerateCode(StreamL, "initialization functions declaration",
		ExtraParameterL);

	StreamL << "void init(void)" << endl;
	StreamL << "{" << endl;

	ComponentsGenerateCode(StreamL, "initialized variables declaration", ExtraParameterL);
	ComponentsGenerateCode(StreamL, "variables initialization",
		ExtraParameterL);
	ComponentsGenerateCode(StreamL, "initialized variables writing",
		ExtraParameterL);

	StreamL << "}" << endl;

	StreamL << "void start_execution(void)" << endl;
	StreamL << "{" << endl;
	
	ComponentsGenerateCode(StreamL, "initialization functions call");

	StreamL << "  init();" << endl;
	StreamL << "  run = 1;" << endl;
	StreamL << "  start_time = gethrtime();" << endl;

	ComponentsGenerateCode(StreamL, "tasks wakeup", ExtraParameterL);

	StreamL << "}" << endl;
	StreamL << "void stop_execution(void)" << endl;
	StreamL << "{" << endl;
	StreamL << "  run = 0;" << endl;

	ComponentsGenerateCode(StreamL, "tasks suspend", ExtraParameterL);
	StreamL << "}" << endl;

	ComponentsGenerateCode(StreamL, "functions declaration",
		ExtraParameterL);

  StreamL << "int init_module(void)" << endl;
  StreamL << "{" << endl;

	StringsSetL.clear();
	ComponentsGenerateCode(StringsSetL, "init_module variables declaration",
		ExtraParameterL);
	copy(StringsSetL.begin(), StringsSetL.end(), ostream_iterator<string>(StreamL, "\n"));

	ComponentsGenerateCode(StreamL, "fifos initialization", ExtraParameterL);
	ComponentsGenerateCode(StreamL, "tasks initialization", ExtraParameterL);

	StringsSetL.clear();
	ComponentsGenerateCode(StringsSetL, "init_module initializations", ExtraParameterL);
	copy(StringsSetL.begin(), StringsSetL.end(), ostream_iterator<string>(StreamL, "\n"));

	ComponentsGenerateCode(StreamL, "before tasks wakeup", ExtraParameterL);
	
	StreamL << "  start_execution();" << endl;
	StreamL << "  return 0;" << endl;
  StreamL << "}" << endl;

  StreamL << "void cleanup_module(void)" << endl;
  StreamL << "{" << endl;
	StreamL << "  stop_execution();" << endl;

	ComponentsGenerateCode(StreamL, "tasks destruction", ExtraParameterL);
	ComponentsGenerateCode(StreamL, "fifos destruction", ExtraParameterL);

  StreamL << "}" << endl;
}
/**  */
long long CDynamicSystem::Time() const
{
	return m_llTime;
}
/**  */
//Del by KDevelop: void CDynamicSystem::Arrange(void* ExtraParameterL = NULL)
//Del by KDevelop: {
//Del by KDevelop: 	// We override the definition of Arrange in CCodeGenerator because it checks Identifier
//Del by KDevelop:   // value
//Del by KDevelop: }
/**  */
bool CDynamicSystem::AcceptsContainer(const CCodeGenerator* pContainer)
{
	AddError(101);
	return false;
}
/**  */
void CDynamicSystem::GenerateCode(ostream& Stream, const CCodeGenerator* pCaller,
	const string& Section, void* pExtraParameter = NULL) const
{
}
/**  */
void CDynamicSystem::ReportErrors(CErrors& Errors)
{
	Errors.Clear();
	AllErrorsReport(Errors);
	AllClearErrors();
}
/**  */
bool CDynamicSystem::ArrangeAll(void* pExtraParameter = NULL)
{
	Components().Arrange(pExtraParameter);
	FixedComponents().Arrange(pExtraParameter);
	return !ErrorsReported();
}
