/***************************************************************************
                          function.cpp  -  description
                             -------------------
    begin                : Tue Feb 15 2000
    copyright            : (C) 2000 by 
    email                : ivan_m_r@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "function.h"

#include "loop.h"
#include "dynamicsystem.h"

CFunction::CFunction()
{
	CodeA = "";
}
CFunction::~CFunction()
{
}
/**  */
string CFunction::Class() const
{
	return "CFunction";
}
/**  */
bool CFunction::DescendantOf(const string& ClassNameL) const
{
	if (ClassNameL == "CFunction")
		return true;
	else
		return CBlock::DescendantOf(ClassNameL);
}
/**  */
void CFunction::GenerateBlockCode(ostream& StreamL) const
{
	StreamL << Code() << endl;
}
/**  */
bool CFunction::SetCode(const string& CodeL)
{
	CodeA = CodeL;

	return true;
}
/**  */
const string& CFunction::Code() const
{
	return CodeA;
}
/**  */
void CFunction::GenerateCode(ostream& Stream, const CCodeGenerator* pCaller,
	const string& Section, void* ExtraParameter = NULL) const
{
	if (pCaller->DescendantOf("CDynamicSystem"))
	{
		if (Section == "initialization functions declaration")
			GenerateInitializationFunctions(Stream, ExtraParameter);
		else
			if (Section == "initialization functions call")
				GenerateInitializationCall(Stream, ExtraParameter);
			else
				if (Section == "functions declaration")
					GenerateFunctions(Stream, ExtraParameter);
	}
	else
		if (pCaller->DescendantOf("CLoop"))
			if (Section == "processes call")
				GenerateCall(Stream, ExtraParameter);
}
/**  */
void CFunction::GenerateFunctions(ostream& StreamL, void* ExtraParameterL = NULL) const
{
	CStringsSet StringsSetL;

	StreamL << "void " << Identifier() << "(void)" <<	endl;
	StreamL << "{" << endl;

	StringsSetL.clear();
	Container()->Container()->ComponentsGenerateCode(StringsSetL, this,
		"local variables declaration", ExtraParameterL);
	copy(StringsSetL.begin(), StringsSetL.end(), ostream_iterator<string>(StreamL, "\n"));

	Container()->Container()->ComponentsGenerateCode(StreamL, this,
		"local system variable declarations", ExtraParameterL);
	Container()->ComponentsGenerateCode(StreamL, this,
		"local loop variable declarations", ExtraParameterL);
	ComponentsGenerateCode(StreamL, "local declarations", ExtraParameterL);

	Container()->Container()->ComponentsGenerateCode(StreamL, this,
		"system variables reading", ExtraParameterL);
	Container()->ComponentsGenerateCode(StreamL, this, "loop variables reading",
		ExtraParameterL);
	ComponentsGenerateCode(StreamL, "inputs reading", ExtraParameterL);

	GenerateBlockCode(StreamL);

	ComponentsGenerateCode(StreamL, "outputs writing", ExtraParameterL);
	Container()->ComponentsGenerateCode(StreamL, this, "loop variables writing",
		ExtraParameterL);
	Container()->Container()->ComponentsGenerateCode(StreamL, this,
		"system variables writing", ExtraParameterL);

	StreamL << "}" << endl;
}
/**  */
void CFunction::GenerateInitializationCall(ostream& Stream,
	void* ExtraParameter = NULL) const
{
	Stream << "  init_" << Container()->Identifier() << "_" << Name() << "();" << endl;
}
/**  */
void CFunction::GenerateInitializationFunctions(ostream& Stream,
	void* ExtraParameter = NULL) const
{
	Stream << "void init_" << Container()->Identifier() << "_" << Name() << "(void)" << endl;
	Stream << "{" << endl;

	ComponentsGenerateCode(Stream, "initialized variables declaration", ExtraParameter);
	ComponentsGenerateCode(Stream, "variables initialization", ExtraParameter);
	ComponentsGenerateCode(Stream, "initialized variables writing", ExtraParameter);

	Stream << "}" << endl;
}
/**  */
