/***************************************************************************
                          functionvariable.cpp  -  description
                             -------------------
    begin                : Mon Jul 10 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivan_m_r@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "functionvariable.h"

#include "loop.h"

CFunctionVariable::CFunctionVariable(){
}
CFunctionVariable::~CFunctionVariable(){
}
/**  */
bool CFunctionVariable::DescendantOf(const string& ClassName) const
{
	if (ClassName == "CFunctionVariable")
		return true;
	else
		return CVariable::DescendantOf(ClassName);
}
/**  */
bool CFunctionVariable::AcceptsContainer(const CCodeGenerator* pContainer)
{
	if (pContainer->DescendantOf("CFunction"))
	{
		SetContainer(pContainer);
		return true;
	}
	else
	{
		AddError(101);
		return false;
	}
}
/**  */
string CFunctionVariable::Class() const
{
	return "CFunctionVariable";
}
/**  */
void CFunctionVariable::GenerateCode(ostream& Stream, const CCodeGenerator* pCaller,
	const string& Section, void* pExtraParameter = NULL) const
{
	if (pCaller->DescendantOf("CDynamicSystem"))
	{
		if (Section == "global variables declaration")
			GenerateGlobalDeclarations(Stream, pExtraParameter);
	}
	else
		if (pCaller->DescendantOf("CFunction"))
			if (Section == "local declarations")
				GenerateLocalDeclarations(Stream, pExtraParameter);
			else			
				if (Section == "outputs writing")
					GenerateGlobalWriting(Stream, pExtraParameter);
}
/**  */
bool CFunctionVariable::SetName(const string& Name)
{
	SetLocalIdentifier(Name);

	if (IllegalCIdentifier(Name))
	{
		AddError(600);
		return false;
	}	
	else
	{
		return true;
	}
}
/**  */
bool CFunctionVariable::SetType(const string& Type)
{
	CVariable::SetType(Type);

	if (IllegalCIdentifier(Type))
	{
		AddError(602);
		return false;
	}	
	else
		return true;
}
/**  */
const CFunction* CFunctionVariable::Container() const
{
	return (CFunction*)CCodeGenerator::Container();
}
/**  */
void CFunctionVariable::Arrange(void* = NULL)
{
	if (IllegalCIdentifier(LocalIdentifier()))
		AddError(600);
	else
	{
		SetIdentifier("functionvar_" + Container()->Container()->Identifier() + "_" +
			Container()->Name() + "_" + LocalIdentifier());
	}
}
