/***************************************************************************
                          rtfifo.cpp  -  description
                             -------------------
    begin                : Tue May 23 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivan_m_r@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "rtfifo.h"

CRTFifo::CRTFifo()
{
	BufferA = "";
	BufferSizeA = "";
	m_Number = "";
	m_Size = "";
}
CRTFifo::~CRTFifo()
{
}
/**  */
void CRTFifo::SetNumber(const string& Number)
{
	m_Number = Number;
}
/**  */
const string& CRTFifo::Number() const
{
	return m_Number;
}
/**  */
void CRTFifo::SetSize(const string& Size)
{
	m_Size = Size;
}
/**  */
const string& CRTFifo::Size() const
{
	return m_Size;
}
/**  */
void CRTFifo::GenerateDeclaration(ostream& StreamL, void* ExtraParameterL = NULL) const
{
  StreamL << "int " << Identifier() << ";" << endl;
}
/**  */
void CRTFifo::GenerateInitialization(ostream& StreamL, void* ExtraParameterL = NULL) const
{
	StreamL << "  rtf_destroy(" << Number() << ");" << endl;
	StreamL << "  rtf_create(" << Number() << ", " << Size() << ");" << endl;
	StreamL << "  " << Identifier() << " = open(\"/dev/rtf" << Number() <<
		"\", O_NONBLOCK + ";
	if (Mode() == CRTFifo::WRITE)
		StreamL << "O_WRONLY";
	else
		StreamL << "O_RDONLY";
	StreamL << ");" << endl;
}
/**  */
void CRTFifo::GenerateDestruction(ostream& StreamL, void* ExtraParameterL = NULL) const
{
	StreamL << "  close(" << Identifier() << ");" << endl;
	StreamL << "  rtf_destroy(" << Number() << ");" << endl;
}
/**  */
const string& CRTFifo::Buffer() const
{
	return BufferA;
}
/**  */
void CRTFifo::SetBuffer(const string& BufferL)
{
	BufferA = BufferL;
}
/**  */
const string& CRTFifo::BufferSize() const
{
	return BufferSizeA;
}
/**  */
void CRTFifo::SetBufferSize(const string& SizeL)
{
	BufferSizeA = SizeL;
}
/**  */
//Del by KDevelop: string CRTFifo::Class() const
//Del by KDevelop: {
//Del by KDevelop: 	return "CRTFifo";
//Del by KDevelop: }
/**  */
bool CRTFifo::DescendantOf(const string& ClassNameL) const
{
	if (ClassNameL == "CRTFifo")
		return true;
	else
		return CCodeGenerator::DescendantOf(ClassNameL);
}
/**  */
void CRTFifo::GenerateFifoWriting(ostream& StreamL, void *ExtraParameterL = NULL) const
{
	StreamL << "      write(" << Identifier() << ", " << Buffer() << ", " <<
		BufferSize() << ");" << endl;
}
/**  */
void CRTFifo::GenerateFifoReading(ostream& StreamL, void *ExtraParameterL = NULL) const
{
	StreamL << "      read(" << Identifier() << ", " << Buffer() << ", " << BufferSize() <<
		");" << endl;
}
/**  */
void CRTFifo::GenerateIncludes(CStringsSet& StringsSetL,
	void* ExtraParameterL = NULL) const
{
	StringsSetL.insert("#include <unistd.h>");
	StringsSetL.insert("#include <rtl_fifo.h>");
	StringsSetL.insert("#include <sys/types.h>");
	StringsSetL.insert("#include <sys/stat.h>");
	// StringsSetL.insert("#include <fcntl.h>");		Produces a parse error in such file
}
/**  */
void CRTFifo::SetMode(EMode Mode)
{
	m_Mode = Mode;
}
/**  */
CRTFifo::EMode CRTFifo::Mode() const
{
	return m_Mode;
}
